import Component from '@glimmer/component';
import { tracked } from '@glimmer/tracking';

export default class CurrentValueDotComponent extends Component {
  @tracked svg;

  // CONFIG
  get color() {
    return this.args.color || 'black';
  }

  get radius() {
    return this.args.radius || 2.5;
  }

  get yKey() {
    return this.args.yKey || 'SUM';
  }

  get xKey() {
    return this.args.xKey || 'OCCURED';
  }

  // COMPUTED
  get noData() {
    return !this.args.dataset || !this.args.dataset.length;
  }

  get datapoint() {
    return this.noData
      ? null
      : this.args.dataset[this.args.dataset.length - 1];
  }

  get dimensions() {
    return this.svg ? this.svg.getBoundingClientRect() : {};
  }

  findSvg = (el) => {
    this.svg = el.closest('svg');
  }

  // Returns x, y coordinates of dot
  // returns {x: , y: }
  get location() {
    if (this.noData || !this.svg) return { x: -100, y: -100 };
    return {
      x: this.getClampedX(),
      y: this.getClampedY()
    }
  }

  // Get x location but shift so that its always inside of chart
  getClampedX() {
    const xValue = new Date(this.datapoint[this.xKey]);
    const xLoc = this.args.xScale(xValue);
    // if ( xLoc < 0 + this.radius ) {
    //   return this.radius - 0.2;
    // } else if (xLoc > this.dimensions.width - this.radius) {
    //   return this.dimensions.width - this.radius + 0.2;
    // } else {
      return xLoc;
    // }
  }

  // Get y location but shift so thats its always inside of chart (not cut off)
  getClampedY() {
    const yValue = this.datapoint[this.yKey];
    const yLoc = this.args.yScale(yValue);
    const xValue = new Date(this.datapoint[this.xKey]);
    const xLoc = this.args.xScale(xValue);
    const xIsAdjusted = xLoc > this.dimensions.width - this.radius;

    let adjustedYLoc = yLoc

    // if (xIsAdjusted) {
    //   // if slope = (y2 - y1) / (x1 - x2) aka rise over run
    //   // then we know (y2 - y3) / (x2 - x3) = slope (where 3 is the shifted point on that line) and x3 is shifted x amount, aka width - radius
    //   // y3 = y2 - slope * (x2 - (width - radius))
    //   const xdifference = xLoc - (this.dimensions.width - this.radius);
    //   const diff = this.findSlope() * xdifference;
    //   const newY = yLoc - diff;
    //   return newY
    // }  else if (yLoc < 0 + this.radius) {
    //   return this.radius;
    // } else if (yLoc > this.dimensions.height - this.radius) {
    //   return this.dimensions.height - this.radius;
    // } else {
      return yLoc;
    // }
  }

  findSlope() {
    if (!this.args.dataset || this.args.dataset.length < 2) return 1;
    const datapointTwo = this.args.dataset[this.args.dataset.length - 2];
    const x1 = this.args.xScale(new Date(datapointTwo[this.xKey]));
    const x2 = this.args.xScale(new Date(this.datapoint[this.xKey]));
    const y1 = this.args.yScale(datapointTwo[this.yKey]);
    const y2 = this.args.yScale(this.datapoint[this.yKey]);
    return (y2 - y1) / (x2 - x1);
  }
}
